from . import polite


@polite
def test_get_movie_discover(tmdb):
    assert tmdb.get_movie_discover() is not None


@polite
def test_get_tv_discover(tmdb):
    assert tmdb.get_tv_discover() is not None



from .base import ENDPOINT, any_required_kwargs, process_response


class SearchMixin:
    @any_required_kwargs(["query"])
    @process_response
    def get_company_search(self, **kwargs):
        """
        GET /search/company
        """

        url = f"{ENDPOINT}/3/search/company"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["query"])
    @process_response
    def get_collection_search(self, **kwargs):
        """
        GET /search/collection
        """

        url = f"{ENDPOINT}/3/search/collection"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["query"])
    @process_response
    def get_keyword_search(self, **kwargs):
        """
        GET /search/keyword
        """

        url = f"{ENDPOINT}/3/search/keyword"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["query"])
    @process_response
    def get_movie_search(self, **kwargs):
        """
        GET /search/movie
        """

        url = f"{ENDPOINT}/3/search/movie"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["query"])
    @process_response
    def get_multi_search(self, **kwargs):
        """
        GET /search/multi
        """

        url = f"{ENDPOINT}/3/search/multi"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["query"])
    @process_response
    def get_person_search(self, **kwargs):
        """
        GET /search/person
        """

        url = f"{ENDPOINT}/3/search/person"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["query"])
    @process_response
    def get_tv_search(self, **kwargs):
        """
        GET /search/tv
        """

        url = f"{ENDPOINT}/3/search/tv"
        return self.make_request("GET", url, kwargs)



from . import polite


@polite
def test_get_movie_genre_list(tmdb):
    assert tmdb.get_movie_genre_list() is not None


@polite
def test_get_tv_genre_list(tmdb):
    assert tmdb.get_tv_genre_list() is not None



from .base import ENDPOINT, process_response


class CreditsMixin:
    @process_response
    def get_credit_details(self, credit_id, **kwargs):
        """
        GET /credit/{credit_id}
        """

        url = f"{ENDPOINT}/3/credit/{credit_id}"
        return self.make_request("GET", url, kwargs)



import os

import pytest

from tmdb.three import TMDb


@pytest.fixture(scope="module")
def tmdb():
    api_key = os.getenv("api_key", None)

    return TMDb(api_key) if api_key else None


@pytest.fixture(scope="module")
def session_id(tmdb):
    username = os.getenv("username")
    password = os.getenv("password")
    request_token = tmdb.create_request_token().get("request_token")

    tmdb.create_session_with_login(username, password, request_token)

    session_id = tmdb.create_session(request_token).get("session_id")

    yield session_id

    tmdb.delete_session(session_id)


@pytest.fixture(scope="module")
def account_id(tmdb, session_id):
    return tmdb.get_account_details(session_id=session_id).get("id")



from . import polite

network_id = 16  # CBS


@polite
def test_get_network_details(tmdb):
    assert tmdb.get_network_details(network_id) is not None


@polite
def test_get_network_alternative_names(tmdb):
    assert tmdb.get_network_alternative_names(network_id) is not None


@polite
def test_get_network_images(tmdb):
    assert tmdb.get_network_images(network_id) is not None



import pytest

from . import polite


tv_id = 1418  # The Big Bang Theory
season_number = 12


@polite
def test_get_tv_season_details(tmdb):
    assert tmdb.get_tv_season_details(tv_id, season_number) is not None


@polite
@pytest.mark.parametrize("is_error", [True, False])
def test_get_tv_season_account_states(tmdb, session_id, is_error):
    if is_error:
        with pytest.raises(RuntimeError):
            assert tmdb.get_tv_season_account_states(tv_id, season_number)
    else:
        assert (
            tmdb.get_tv_season_account_states(
                tv_id, season_number, session_id=session_id
            )
            is not None
        )


@polite
def test_get_tv_season_changes(tmdb):
    season_id = tmdb.get_tv_season_details(tv_id, season_number).get("id")
    assert tmdb.get_tv_season_changes(season_id) is not None


@polite
def test_get_tv_season_credits(tmdb):
    assert tmdb.get_tv_season_credits(tv_id, season_number) is not None


@polite
def test_get_tv_season_external_ids(tmdb):
    assert tmdb.get_tv_season_external_ids(tv_id, season_number) is not None


@polite
def test_get_tv_season_images(tmdb):
    assert tmdb.get_tv_season_images(tv_id, season_number) is not None


@polite
def test_get_tv_season_videos(tmdb):
    assert tmdb.get_tv_season_videos(tv_id, season_number) is not None



from .base import ENDPOINT, any_required_kwargs, process_response


class TVEpisodesMixin:
    @process_response
    def get_tv_episode_details(self, tv_id, season_number, episode_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/episode/{episode_number}
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_episode_changes(self, episode_id, **kwargs):
        """
        GET /tv/episode/{episode_id}/changes
        """

        url = f"{ENDPOINT}/3/tv/episode/{episode_id}/changes"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_tv_episode_account_states(
        self, tv_id, season_number, episode_number, **kwargs
    ):
        """
        GET /tv/{tv_id}/season/{season_number}/episode/{episode_number}/account_states
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/account_states"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_episode_credits(self, tv_id, season_number, episode_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/episode/{episode_number}/credits
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/credits"
        return self.make_request("GET", url, kwargs)

    @process_response
    # fmt: off
    def get_tv_episode_external_ids(self, tv_id, season_number, episode_number, **kwargs):
    # fmt: on
        """
        GET /tv/{tv_id}/season/{season_number}/episode/{episode_number}/external_ids
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/external_ids"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_episode_images(self, tv_id, season_number, episode_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/episode/{episode_number}/images
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/images"
        return self.make_request("GET", url, kwargs)

    @process_response
    # fmt: off
    def get_tv_episode_translations(self, tv_id, season_number, episode_number, **kwargs):
    # fmt: on
        """
        GET /tv/{tv_id}/season/{season_number}/episode/{episode_number}/translations
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/translations"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"], ["guest_session_id"])
    @process_response
    # fmt: off
    def post_tv_episode_rating(self, tv_id, season_number, episode_number, rating, **kwargs):
    # fmt: on
        """
        POST /tv/{tv_id}/season/{season_number}/episode/{episode_number}/rating
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/rating"
        payload = {"value": rating}
        return self.make_request("POST", url, kwargs, payload)

    @any_required_kwargs(["session_id"], ["guest_session_id"])
    @process_response
    def delete_tv_episode_rating(self, tv_id, season_number, episode_number, **kwargs):
        """
        DELETE /tv/{tv_id}/season/{season_number}/episode/{episode_number}/rating
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/rating"
        return self.make_request("DELETE", url, kwargs)

    @process_response
    def get_tv_episode_videos(self, tv_id, season_number, episode_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/episode/{episode_number}/videos
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/videos"
        return self.make_request("GET", url, kwargs)



from . import polite

person_id = 192  # Morgan Freeman


@polite
def test_get_person_details(tmdb):
    assert tmdb.get_person_details(person_id) is not None


@polite
def test_get_person_changes(tmdb):
    assert tmdb.get_person_changes(person_id) is not None


@polite
def test_get_person_movie_credits(tmdb):
    assert tmdb.get_person_movie_credits(person_id) is not None


@polite
def test_get_person_tv_credits(tmdb):
    assert tmdb.get_person_tv_credits(person_id) is not None


@polite
def test_get_person_combined_credits(tmdb):
    assert tmdb.get_person_combined_credits(person_id) is not None


@polite
def test_get_person_external_ids(tmdb):
    assert tmdb.get_person_external_ids(person_id) is not None


@polite
def test_get_person_images(tmdb):
    assert tmdb.get_person_images(person_id) is not None


@polite
def test_get_person_tagged_images(tmdb):
    assert tmdb.get_person_tagged_images(person_id) is not None


@polite
def test_get_person_translations(tmdb):
    assert tmdb.get_person_translations(person_id) is not None


@polite
def test_get_latest_person(tmdb):
    assert tmdb.get_latest_person() is not None


@polite
def test_get_popular_persons(tmdb):
    assert tmdb.get_popular_persons() is not None



import pytest

from . import polite

externals = [
    ("imdb_id", "tt0903747"),
    ("freebase_mid", "/m/03d34x8"),
    ("freebase_id", "en/breaking_bad"),
    ("tvdb_id", 81189),
    ("tvrage_id", 18164),
]


@polite
@pytest.mark.parametrize("is_error", [True, False])
@pytest.mark.parametrize("external_source, external_id", externals)
def test_find_by_id(tmdb, is_error, external_source, external_id):
    if is_error:
        with pytest.raises(RuntimeError):
            assert tmdb.find_by_id(external_id)
    else:
        assert tmdb.find_by_id(external_id, external_source=external_source) is not None



from .base import ENDPOINT, any_required_kwargs, process_response


class FindMixin:
    @any_required_kwargs(["external_source"])
    @process_response
    def find_by_id(self, external_id, **kwargs):
        """
        GET /find/{external_id}
        """

        url = f"{ENDPOINT}/3/find/{external_id}"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class AuthenticationMixin:
    @process_response
    def create_guest_session(self, **kwargs):
        """
        GET /authentication/guest_session/new
        """

        url = f"{ENDPOINT}/3/authentication/guest_session/new"
        return self.make_request("GET", url, kwargs)

    @process_response
    def create_request_token(self, **kwargs):
        """
        GET /authentication/token/new
        """

        url = f"{ENDPOINT}/3/authentication/token/new"
        return self.make_request("GET", url, kwargs)

    @process_response
    def create_session(self, request_token, **kwargs):
        """
        POST /authentication/session/new
        """

        url = f"{ENDPOINT}/3/authentication/session/new"
        payload = {"request_token": request_token}
        return self.make_request("POST", url, kwargs, payload)

    @process_response
    def create_session_with_login(self, username, password, request_token, **kwargs):
        """
        POST /authentication/token/validate_with_login
        """
        url = f"{ENDPOINT}/3/authentication/token/validate_with_login"
        payload = {
            "username": username,
            "password": password,
            "request_token": request_token,
        }
        return self.make_request("POST", url, kwargs, payload)

    def create_session_from_v4_access_token(self, **kwargs):
        """
        POST /authentication/session/convert/4
        """

        raise NotImplementedError

    @process_response
    def delete_session(self, session_id, **kwargs):
        """
        DELETE /authentication/session
        """

        url = f"{ENDPOINT}/3/authentication/session"
        payload = {"session_id": session_id}
        return self.make_request("DELETE", url, kwargs, payload)



import pytest

from . import polite

media_type, media_id = "movie", 550


@polite
def test_get_account_details(tmdb, session_id):
    assert tmdb.get_account_details(session_id=session_id) is not None


@polite
def test_get_account_lists(tmdb, account_id, session_id):
    assert tmdb.get_account_lists(account_id, session_id=session_id) is not None


@polite
def test_get_account_favorite_movies(tmdb, account_id, session_id):
    assert (
        tmdb.get_account_favorite_movies(account_id, session_id=session_id) is not None
    )


@polite
def test_get_account_favorite_tvs(tmdb, account_id, session_id):
    assert tmdb.get_account_favorite_tvs(account_id, session_id=session_id) is not None


@polite
@pytest.mark.parametrize("favorite", [True, False])
def test_mark_as_favorite(tmdb, account_id, session_id, favorite):
    assert (
        tmdb.mark_as_favorite(
            account_id, media_type, media_id, favorite, session_id=session_id
        )
        is not None
    )


@polite
def test_get_account_rated_movies(tmdb, account_id, session_id):
    assert tmdb.get_account_rated_movies(account_id, session_id=session_id) is not None


@polite
def test_get_account_rated_tvs(tmdb, account_id, session_id):
    assert tmdb.get_account_rated_tvs(account_id, session_id=session_id) is not None


@polite
def test_get_account_rated_tv_episodes(tmdb, account_id, session_id):
    assert (
        tmdb.get_account_rated_tv_episodes(account_id, session_id=session_id)
        is not None
    )


@polite
def test_get_account_movie_watchlist(tmdb, account_id, session_id):
    assert (
        tmdb.get_account_movie_watchlist(account_id, session_id=session_id) is not None
    )


@polite
def test_get_account_tv_watchlist(tmdb, account_id, session_id):
    assert tmdb.get_account_tv_watchlist(account_id, session_id=session_id) is not None


@polite
@pytest.mark.parametrize("watchlist", [True, False])
def test_add_to_watchlist(tmdb, account_id, session_id, watchlist):
    assert (
        tmdb.add_to_watchlist(
            account_id, media_type, media_id, watchlist, session_id=session_id
        )
        is not None
    )



import requests

from .account import AccountMixin
from .authentication import AuthenticationMixin
from .certifications import CertificationsMixin
from .changes import ChangesMixin
from .collections import CollectionsMixin
from .companies import CompaniesMixin
from .configuration import ConfigurationMixin
from .credits import CreditsMixin
from .discover import DiscoverMixin
from .find import FindMixin
from .genres import GenresMixin
from .guest_sessions import GuestSessionsMixin
from .keywords import KeywordsMixin
from .lists import ListsMixin
from .movies import MoviesMixin
from .networks import NetworksMixin
from .people import PeopleMixin
from .search import SearchMixin
from .reviews import ReviewsMixin
from .trending import TrendingMixin
from .tv import TVsMixin
from .tv_episodes import TVEpisodesMixin
from .tv_episode_groups import TVEpisodeGroupsMixin
from .tv_seasons import TVSeasonsMixin

mixins = [
    AccountMixin,
    AuthenticationMixin,
    CertificationsMixin,
    ChangesMixin,
    CollectionsMixin,
    CompaniesMixin,
    ConfigurationMixin,
    CreditsMixin,
    DiscoverMixin,
    FindMixin,
    GenresMixin,
    GuestSessionsMixin,
    KeywordsMixin,
    ListsMixin,
    MoviesMixin,
    NetworksMixin,
    PeopleMixin,
    ReviewsMixin,
    SearchMixin,
    TrendingMixin,
    TVEpisodeGroupsMixin,
    TVEpisodesMixin,
    TVSeasonsMixin,
    TVsMixin,
]


class TMDb(*mixins):
    def __init__(self, api_key, **kwargs):
        self.api_key = api_key
        self.options = kwargs
        self.session = requests.Session()
        self.session.headers.update({"Content-Type": "application/json;charset=utf-8"})

    def make_request(self, method, url, request_params, payload=None):
        params = {"api_key": self.api_key}
        params.update(self.options)
        params.update(request_params)

        # https://developers.themoviedb.org/3/getting-started/image-languages
        # convert keyword argument: (..., include_image_language=["en", "null"])
        # into url param: ...&include_image_language=en,null
        for key in params:
            if type(params[key]) is list:
                params[key] = ",".join(params[key])

        return self.session.request(method, url, params=params, json=payload)

    def set_options(self, **kwargs):
        self.options.update(kwargs)

    def reset_options(self):
        self.options = {}



from . import polite

tv_episode_group_id = "5b1aa75ac3a368490e016641"


@polite
def test_get_tv_episode_group_details(tmdb):
    assert tmdb.get_tv_episode_group_details(tv_episode_group_id) is not None



from .base import ENDPOINT, any_required_kwargs, process_response


class AccountMixin:
    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_details(self, **kwargs):
        """
        GET /account
        """

        url = f"{ENDPOINT}/3/account"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_lists(self, account_id, **kwargs):
        """
        GET /account/{account_id}/lists
        """

        url = f"{ENDPOINT}/3/account/{account_id}/lists"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_favorite_movies(self, account_id, **kwargs):
        """
        GET /account/{account_id}/favorite/movies
        """

        url = f"{ENDPOINT}/3/account/{account_id}/favorite/movies"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_favorite_tvs(self, account_id, **kwargs):
        """
        GET /account/{account_id}/favorite/tv
        """

        url = f"{ENDPOINT}/3/account/{account_id}/favorite/tv"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def mark_as_favorite(self, account_id, media_type, media_id, favorite, **kwargs):
        """
        POST /account/{account_id}/favorite
        """

        url = f"{ENDPOINT}/3/account/{account_id}/favorite"
        payload = {
            "media_type": media_type,
            "media_id": media_id,
            "favorite": str(favorite).lower(),
        }
        return self.make_request("POST", url, kwargs, payload)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_rated_movies(self, account_id, **kwargs):
        """
        GET /account/{account_id}/rated/movies
        """

        url = f"{ENDPOINT}/3/account/{account_id}/rated/movies"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_rated_tvs(self, account_id, **kwargs):
        """
        GET /account/{account_id}/rated/tv
        """

        url = f"{ENDPOINT}/3/account/{account_id}/rated/tv"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_rated_tv_episodes(self, account_id, **kwargs):
        """
        GET /account/{account_id}/rated/tv/episodes
        """

        url = f"{ENDPOINT}/3/account/{account_id}/rated/tv/episodes"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_movie_watchlist(self, account_id, **kwargs):
        """
        GET /account/{account_id}/watchlist/movies
        """

        url = f"{ENDPOINT}/3/account/{account_id}/watchlist/movies"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_account_tv_watchlist(self, account_id, **kwargs):
        """
        GET /account/{account_id}/watchlist/tv
        """

        url = f"{ENDPOINT}/3/account/{account_id}/watchlist/tv"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def add_to_watchlist(self, account_id, media_type, media_id, watchlist, **kwargs):
        """
        POST /account/{account_id}/watchlist
        """

        url = f"{ENDPOINT}/3/account/{account_id}/watchlist"
        payload = {
            "media_type": media_type,
            "media_id": media_id,
            "watchlist": str(watchlist).lower(),
        }
        return self.make_request("POST", url, kwargs, payload)



from .base import ENDPOINT, process_response


class TVEpisodeGroupsMixin:
    @process_response
    def get_tv_episode_group_details(self, tv_episode_group_id, **kwargs):
        """
        GET /tv/episode_group/{id}
        """

        url = f"{ENDPOINT}/3/tv/episode_group/{tv_episode_group_id}"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class DiscoverMixin:
    @process_response
    def get_movie_discover(self, **kwargs):
        """
        GET /discover/movie
        """

        url = f"{ENDPOINT}/3/discover/movie"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_discover(self, **kwargs):
        """
        GET /discover/tv
        """

        url = f"{ENDPOINT}/3/discover/tv"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class ConfigurationMixin:
    @process_response
    def get_api_configuration(self, **kwargs):
        """
        GET /configuration
        """

        url = f"{ENDPOINT}/3/configuration"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_countries(self, **kwargs):
        """
        GET /configuration/countries
        """

        url = f"{ENDPOINT}/3/configuration/countries"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_jobs(self, **kwargs):
        """
        GET /configuration/jobs
        """

        url = f"{ENDPOINT}/3/configuration/jobs"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_languages(self, **kwargs):
        """
        GET /configuration/languages
        """

        url = f"{ENDPOINT}/3/configuration/languages"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_primary_translations(self, **kwargs):
        """
        GET /configuration/primary_translations
        """

        url = f"{ENDPOINT}/3/configuration/primary_translations"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_timezones(self, **kwargs):
        """
        GET /configuration/timezones
        """

        url = f"{ENDPOINT}/3/configuration/timezones"
        return self.make_request("GET", url, kwargs)



import pytest

from . import polite


tv_id = 1418  # The Big Bang Theory
season_number = 12
episode_number = 1


@polite
def test_get_tv_episode_details(tmdb):
    assert tmdb.get_tv_episode_details(tv_id, season_number, episode_number) is not None


@polite
def test_get_tv_episode_changes(tmdb):
    episode_id = tmdb.get_tv_episode_details(tv_id, season_number, episode_number).get(
        "id"
    )
    assert tmdb.get_tv_episode_changes(episode_id) is not None


@polite
@pytest.mark.parametrize("with_guest_session", [True, False])
def test_post_tv_episode_rating(tmdb, with_guest_session):
    rating = 10

    if with_guest_session:
        guest_session_id = tmdb.create_guest_session().get("guest_session_id")
        assert tmdb.post_tv_episode_rating(
            tv_id,
            season_number,
            episode_number,
            rating,
            guest_session_id=guest_session_id,
        )
    else:
        with pytest.raises(RuntimeError):
            assert tmdb.post_tv_episode_rating(
                tv_id, season_number, episode_number, rating
            )


@polite
@pytest.mark.parametrize("with_guest_session", [True, False])
def test_delete_tv_episode_rating(tmdb, with_guest_session):
    if with_guest_session:
        guest_session_id = tmdb.create_guest_session().get("guest_session_id")
        assert tmdb.delete_tv_episode_rating(
            tv_id, season_number, episode_number, guest_session_id=guest_session_id
        )
    else:
        with pytest.raises(RuntimeError):
            assert tmdb.delete_tv_episode_rating(tv_id, season_number, episode_number)


@polite
def test_get_tv_episode_credits(tmdb):
    assert tmdb.get_tv_episode_credits(tv_id, season_number, episode_number) is not None


@polite
def test_get_tv_episode_external_ids(tmdb):
    assert (
        tmdb.get_tv_episode_external_ids(tv_id, season_number, episode_number)
        is not None
    )


@polite
def test_get_tv_episode_images(tmdb):
    assert tmdb.get_tv_episode_images(tv_id, season_number, episode_number) is not None


@polite
def test_get_tv_episode_translations(tmdb):
    assert (
        tmdb.get_tv_episode_translations(tv_id, season_number, episode_number)
        is not None
    )


@polite
def test_get_tv_episode_videos(tmdb):
    assert tmdb.get_tv_episode_videos(tv_id, season_number, episode_number) is not None



from .base import ENDPOINT, any_required_kwargs, process_response


class TVsMixin:
    @process_response
    def get_tv_details(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_tv_account_states(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/account_states
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/account_states"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_alternative_titles(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/alternative_titles
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/alternative_titles"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_changes(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/changes
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/changes"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_content_ratings(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/content_ratings
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/content_ratings"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_credits(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/credits
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/credits"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_episode_groups(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/episode_groups
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/episode_groups"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_external_ids(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/external_ids
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/external_ids"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_images(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/images
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/images"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_keywords(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/keywords
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/keywords"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_recommendations(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/recommendations
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/recommendations"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_reviews(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/reviews
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/reviews"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_screened_theatrically(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/screened_theatrically
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/screened_theatrically"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_similar(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/similar
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/similar"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_translations(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/translations
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/translations"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_videos(self, tv_id, **kwargs):
        """
        GET /tv/{tv_id}/videos
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/videos"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"], ["guest_session_id"])
    @process_response
    def post_tv_rating(self, tv_id, rating, **kwargs):
        """
        POST /tv/{tv_id}/rating
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/rating"
        payload = {"value": rating}
        return self.make_request("POST", url, kwargs, payload)

    @any_required_kwargs(["session_id"], ["guest_session_id"])
    @process_response
    def delete_tv_rating(self, tv_id, **kwargs):
        """
        DELETE /tv/{tv_id}/rating
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/rating"
        return self.make_request("DELETE", url, kwargs)

    @process_response
    def get_latest_tvs(self, **kwargs):
        """
        GET /tv/latest
        """

        url = f"{ENDPOINT}/3/tv/latest"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tvs_airing_today(self, **kwargs):
        """
        GET /tv/airing_today
        """

        url = f"{ENDPOINT}/3/tv/airing_today"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tvs_on_the_air(self, **kwargs):
        """
        GET /tv/on_the_air
        """

        url = f"{ENDPOINT}/3/tv/on_the_air"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_popular_tvs(self, **kwargs):
        """
        GET /tv/popular
        """

        url = f"{ENDPOINT}/3/tv/popular"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_top_rated_tvs(self, **kwargs):
        """
        GET /tv/top_rated
        """

        url = f"{ENDPOINT}/3/tv/top_rated"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class PeopleMixin:
    @process_response
    def get_person_details(self, person_id, **kwargs):
        """
        GET /person/{person_id}
        """

        url = f"{ENDPOINT}/3/person/{person_id}"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_changes(self, person_id, **kwargs):
        """
        GET /person/{person_id}/changes
        """

        url = f"{ENDPOINT}/3/person/{person_id}/changes"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_movie_credits(self, person_id, **kwargs):
        """
        GET /person/{person_id}/movie_credits
        """

        url = f"{ENDPOINT}/3/person/{person_id}/movie_credits"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_tv_credits(self, person_id, **kwargs):
        """
        GET /person/{person_id}/tv_credits
        """

        url = f"{ENDPOINT}/3/person/{person_id}/tv_credits"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_combined_credits(self, person_id, **kwargs):
        """
        GET /person/{person_id}/combined_credits
        """

        url = f"{ENDPOINT}/3/person/{person_id}/combined_credits"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_external_ids(self, person_id, **kwargs):
        """
        GET /person/{person_id}/external_ids
        """

        url = f"{ENDPOINT}/3/person/{person_id}/external_ids"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_images(self, person_id, **kwargs):
        """
        GET /person/{person_id}/images
        """

        url = f"{ENDPOINT}/3/person/{person_id}/images"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_tagged_images(self, person_id, **kwargs):
        """
        GET /person/{person_id}/tagged_images
        """

        url = f"{ENDPOINT}/3/person/{person_id}/tagged_images"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_translations(self, person_id, **kwargs):
        """
        GET /person/{person_id}/translations
        """

        url = f"{ENDPOINT}/3/person/{person_id}/translations"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_latest_person(self, **kwargs):
        """
        GET /person/latest
        """

        url = f"{ENDPOINT}/3/person/latest"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_popular_persons(self, **kwargs):
        """
        GET /person/popular
        """

        url = f"{ENDPOINT}/3/person/popular"
        return self.make_request("GET", url, kwargs)



import os

import pytest

from . import polite


@polite
def test_create_guest_session(tmdb):
    assert tmdb.create_guest_session() is not None


@polite
def test_create_request_token(tmdb):
    assert tmdb.create_request_token() is not None


@polite
def test_session_workflow_with_login(tmdb):
    username = os.getenv("username")
    password = os.getenv("password")
    request_token = tmdb.create_request_token().get("request_token")

    assert username and password and request_token
    assert tmdb.create_session_with_login(username, password, request_token)

    session_id = tmdb.create_session(request_token).get("session_id")

    assert tmdb.delete_session(session_id) is not None


@polite
def test_create_session_from_v4_access_token(tmdb):
    with pytest.raises(NotImplementedError):
        assert tmdb.create_session_from_v4_access_token()



from .base import ENDPOINT, process_response


class TrendingMixin:
    @process_response
    def get_trending(self, media_type, time_window, **kwargs):
        """
        GET /trending/{media_type}/{time_window}
        """

        url = f"{ENDPOINT}/3/trending/{media_type}/{time_window}"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class KeywordsMixin:
    @process_response
    def get_keyword_details(self, keyword_id, **kwargs):
        """
        GET /keyword/{keyword_id}
        """

        url = f"{ENDPOINT}/3/keyword/{keyword_id}"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_keyword_movies(self, keyword_id, **kwargs):
        """
        GET /keyword/{keyword_id}/movies
        """

        url = f"{ENDPOINT}/3/keyword/{keyword_id}/movies"
        return self.make_request("GET", url, kwargs)



from . import polite


@polite
def test_get_api_configuration(tmdb):
    assert tmdb.get_api_configuration() is not None


@polite
def test_get_countries(tmdb):
    assert tmdb.get_countries() is not None


@polite
def test_get_jobs(tmdb):
    assert tmdb.get_jobs() is not None


@polite
def test_get_languages(tmdb):
    assert tmdb.get_languages() is not None


@polite
def test_get_primary_translations(tmdb):
    assert tmdb.get_primary_translations() is not None


@polite
def test_get_timezones(tmdb):
    assert tmdb.get_timezones() is not None



from .base import ENDPOINT, process_response


class ChangesMixin:
    @process_response
    def get_movie_change_list(self, **kwargs):
        """
        GET /movie/changes
        """

        url = f"{ENDPOINT}/3/movie/changes"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_change_list(self, **kwargs):
        """
        GET /tv/changes
        """

        url = f"{ENDPOINT}/3/tv/changes"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_person_change_list(self, **kwargs):
        """
        GET /person/changes
        """

        url = f"{ENDPOINT}/3/person/changes"
        return self.make_request("GET", url, kwargs)



import pytest

from . import polite

movie_id = 597  # Titanic


@polite
def test_get_movie_details(tmdb):
    assert tmdb.get_movie_details(movie_id) is not None


@polite
@pytest.mark.parametrize("is_error", [True, False])
def test_get_movie_account_states(tmdb, session_id, is_error):
    if is_error:
        with pytest.raises(RuntimeError):
            assert tmdb.get_movie_account_states(movie_id)
    else:
        assert (
            tmdb.get_movie_account_states(movie_id, session_id=session_id) is not None
        )


@polite
def test_get_movie_alternative_titles(tmdb):
    assert tmdb.get_movie_alternative_titles(movie_id) is not None


@polite
def test_get_movie_credits(tmdb):
    assert tmdb.get_movie_credits(movie_id) is not None


@polite
def test_get_movie_changes(tmdb):
    assert tmdb.get_movie_changes(movie_id) is not None


@polite
def test_get_movie_external_ids(tmdb):
    assert tmdb.get_movie_external_ids(movie_id) is not None


@polite
def test_get_movie_images(tmdb):
    assert tmdb.get_movie_images(movie_id) is not None

    tmdb.set_options(include_image_language=["en", "null"])
    assert tmdb.get_movie_images(movie_id) is not None


@polite
def test_get_movie_keywords(tmdb):
    assert tmdb.get_movie_keywords(movie_id) is not None


@polite
def test_get_movie_release_dates(tmdb):
    assert tmdb.get_movie_release_dates(movie_id) is not None


@polite
def test_get_movie_videos(tmdb):
    assert tmdb.get_movie_videos(movie_id) is not None


@polite
def test_get_movie_translations(tmdb):
    assert tmdb.get_movie_translations(movie_id) is not None


@polite
def test_get_movie_recommendations(tmdb):
    assert tmdb.get_movie_recommendations(movie_id) is not None


@polite
def test_get_movie_similar(tmdb):
    assert tmdb.get_movie_similar(movie_id) is not None


@polite
def test_get_movie_reviews(tmdb):
    assert tmdb.get_movie_reviews(movie_id) is not None


@polite
def test_get_movie_lists(tmdb):
    assert tmdb.get_movie_lists(movie_id) is not None


@polite
@pytest.mark.parametrize("is_error", [True, False])
def test_post_movie_rating(tmdb, session_id, is_error):
    if is_error:
        with pytest.raises(RuntimeError):
            assert tmdb.post_movie_rating(movie_id, 10)
    else:
        assert tmdb.post_movie_rating(movie_id, 10, session_id=session_id) is not None


@polite
@pytest.mark.parametrize("is_error", [True, False])
def test_delete_movie_rating(tmdb, session_id, is_error):
    if is_error:
        with pytest.raises(RuntimeError):
            assert tmdb.delete_movie_rating(movie_id)
    else:
        assert tmdb.delete_movie_rating(movie_id, session_id=session_id) is not None


@polite
def test_get_latest_movies(tmdb):
    assert tmdb.get_latest_movies() is not None


@polite
def test_get_now_playing_movies(tmdb):
    assert tmdb.get_now_playing_movies() is not None


@polite
def test_get_popular_movies(tmdb):
    assert tmdb.get_popular_movies() is not None


@polite
def test_get_top_rated_movies(tmdb):
    assert tmdb.get_top_rated_movies() is not None


@polite
def test_get_upcoming_movies(tmdb):
    assert tmdb.get_upcoming_movies() is not None



from .base import ENDPOINT, process_response


class NetworksMixin:
    @process_response
    def get_network_details(self, network_id, **kwargs):
        """
        GET /network/{network_id}
        """

        url = f"{ENDPOINT}/3/network/{network_id}"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_network_alternative_names(self, network_id, **kwargs):
        """
        GET /network/{network_id}/alternative_names
        """

        url = f"{ENDPOINT}/3/network/{network_id}/alternative_names"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_network_images(self, network_id, **kwargs):
        """
        GET /network/{network_id}/images
        """

        url = f"{ENDPOINT}/3/network/{network_id}/images"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class CertificationsMixin:
    @process_response
    def get_movie_certifications(self, **kwargs):
        """
        GET /certification/movie/list
        """

        url = f"{ENDPOINT}/3/certification/movie/list"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_certifications(self, **kwargs):
        """
        GET /certification/tv/list
        """

        url = f"{ENDPOINT}/3/certification/tv/list"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class CompaniesMixin:
    @process_response
    def get_company_details(self, company_id, **kwargs):
        """
        GET /company/{company_id}
        """

        url = f"{ENDPOINT}/3/company/{company_id}"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_company_alternative_names(self, company_id, **kwargs):
        """
        GET /company/{company_id}/alternative_names
        """

        url = f"{ENDPOINT}/3/company/{company_id}/alternative_names"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_company_images(self, company_id, **kwargs):
        """
        GET /company/{company_id}/images
        """

        url = f"{ENDPOINT}/3/company/{company_id}/images"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class ReviewsMixin:
    @process_response
    def get_review_details(self, review_id, **kwargs):
        """
        GET /review/{review_id}
        """

        url = f"{ENDPOINT}/3/review/{review_id}"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class GenresMixin:
    @process_response
    def get_movie_genre_list(self, **kwargs):
        """
        GET /genre/movie/list
        """

        url = f"{ENDPOINT}/3/genre/movie/list"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_genre_list(self, **kwargs):
        """
        GET /genre/tv/list
        """

        url = f"{ENDPOINT}/3/genre/tv/list"
        return self.make_request("GET", url, kwargs)



from itertools import product

import pytest

from . import polite

media_types = ["all", "movie", "tv", "person"]
time_windows = ["day", "week"]


@polite
@pytest.mark.parametrize("media_type, time_window", product(media_types, time_windows))
def test_get_trending(tmdb, media_type, time_window):
    assert tmdb.get_trending(media_type, time_window) is not None


@polite
@pytest.mark.parametrize("media_type, time_window", product(media_types, time_windows))
def test_get_trending_de(tmdb, media_type, time_window):
    tmdb.set_options(language="de", region="de")
    assert tmdb.options
    assert tmdb.get_trending(media_type, time_window) is not None

    tmdb.reset_options()
    assert not tmdb.options
    assert tmdb.get_trending(media_type, time_window) is not None



import pytest

from . import polite


tv_id = 1418  # The Big Bang Theory


@polite
def test_get_tv_details(tmdb):
    assert tmdb.get_tv_details(tv_id) is not None


@polite
@pytest.mark.parametrize("is_error", [True, False])
def test_get_tv_account_states(tmdb, session_id, is_error):
    if is_error:
        with pytest.raises(RuntimeError):
            assert tmdb.get_tv_account_states(tv_id)
    else:
        assert tmdb.get_tv_account_states(tv_id, session_id=session_id) is not None


@polite
def test_get_tv_alternative_titles(tmdb):
    assert tmdb.get_tv_alternative_titles(tv_id) is not None


@polite
def test_get_tv_changes(tmdb):
    assert tmdb.get_tv_changes(tv_id) is not None


@polite
def test_get_tv_content_ratings(tmdb):
    assert tmdb.get_tv_content_ratings(tv_id) is not None


@polite
def test_get_tv_credits(tmdb):
    assert tmdb.get_tv_credits(tv_id) is not None


@polite
def test_get_tv_episode_groups(tmdb):
    assert tmdb.get_tv_episode_groups(tv_id) is not None


@polite
def test_get_tv_external_ids(tmdb):
    assert tmdb.get_tv_external_ids(tv_id) is not None


@polite
def test_get_tv_images(tmdb):
    assert tmdb.get_tv_images(tv_id) is not None


@polite
def test_get_tv_keywords(tmdb):
    assert tmdb.get_tv_keywords(tv_id) is not None


@polite
def test_get_tv_recommendations(tmdb):
    assert tmdb.get_tv_recommendations(tv_id) is not None


@polite
def test_get_tv_reviews(tmdb):
    assert tmdb.get_tv_reviews(tv_id) is not None


@polite
def test_get_tv_screened_theatrically(tmdb):
    assert tmdb.get_tv_screened_theatrically(tv_id) is not None


@polite
def test_get_tv_similar(tmdb):
    assert tmdb.get_tv_similar(tv_id) is not None


@polite
def test_get_tv_translations(tmdb):
    assert tmdb.get_tv_translations(tv_id) is not None


@polite
def test_get_tv_videos(tmdb):
    assert tmdb.get_tv_videos(tv_id) is not None


@polite
@pytest.mark.parametrize("is_error", [True, False])
def test_post_tv_rating(tmdb, session_id, is_error):
    if is_error:
        with pytest.raises(RuntimeError):
            assert tmdb.post_tv_rating(tv_id, 10)
    else:
        assert tmdb.post_tv_rating(tv_id, 10, session_id=session_id) is not None


@polite
@pytest.mark.parametrize("is_error", [True, False])
def test_delete_tv_rating(tmdb, session_id, is_error):
    if is_error:
        with pytest.raises(RuntimeError):
            assert tmdb.delete_tv_rating(tv_id)
    else:
        assert tmdb.delete_tv_rating(tv_id, session_id=session_id) is not None


@polite
def test_get_latest_tvs(tmdb):
    assert tmdb.get_latest_tvs() is not None


@polite
def test_get_tvs_airing_today(tmdb):
    assert tmdb.get_tvs_airing_today() is not None


@polite
def test_get_tvs_on_the_air(tmdb):
    assert tmdb.get_tvs_on_the_air() is not None


@polite
def test_get_popular_tvs(tmdb):
    assert tmdb.get_popular_tvs() is not None


@polite
def test_get_top_rated_tvs(tmdb):
    assert tmdb.get_top_rated_tvs() is not None



import pytest

from . import polite


@polite
def test_get_guest_session_rated_movies(tmdb):
    guest_session_id = tmdb.create_guest_session().get("guest_session_id")
    assert tmdb.get_guest_session_rated_movies(guest_session_id) is not None


@polite
def test_get_guest_session_rated_tvs(tmdb):
    guest_session_id = tmdb.create_guest_session().get("guest_session_id")
    assert tmdb.get_guest_session_rated_tvs(guest_session_id) is not None


@polite
def test_get_guest_session_rated_tv_episodes(tmdb):
    guest_session_id = tmdb.create_guest_session().get("guest_session_id")
    assert tmdb.get_guest_session_rated_tv_episodes(guest_session_id) is not None



from . import polite


@polite
def test_get_movie_change_list(tmdb):
    assert tmdb.get_movie_change_list() is not None


@polite
def test_get_tv_change_list(tmdb):
    assert tmdb.get_tv_change_list() is not None


@polite
def test_get_person_change_list(tmdb):
    assert tmdb.get_person_change_list() is not None



from . import polite

collection_id = 86311  # The Avengers Collection


@polite
def test_get_collection_details(tmdb):
    assert tmdb.get_collection_details(collection_id) is not None


@polite
def test_get_collection_images(tmdb):
    assert tmdb.get_collection_images(collection_id) is not None


@polite
def test_get_collection_translations(tmdb):
    assert tmdb.get_collection_translations(collection_id) is not None



from .base import ENDPOINT, process_response


class CollectionsMixin:
    @process_response
    def get_collection_details(self, collection_id, **kwargs):
        """
        GET /collection/{collection_id}
        """

        url = f"{ENDPOINT}/3/collection/{collection_id}"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_collection_images(self, collection_id, **kwargs):
        """
        GET /collection/{collection_id}/images
        """

        url = f"{ENDPOINT}/3/collection/{collection_id}/images"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_collection_translations(self, collection_id, **kwargs):
        """
        GET /collection/{collection_id}/translations
        """

        url = f"{ENDPOINT}/3/collection/{collection_id}/translations"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, any_required_kwargs, process_response


class ListsMixin:
    @process_response
    def get_list_details(self, list_id, **kwargs):
        """
        GET /list/{list_id}
        """

        url = f"{ENDPOINT}/3/list/{list_id}"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["movie_id"])
    @process_response
    def get_list_item_status(self, list_id, **kwargs):
        """
        GET /list/{list_id}/item_status
        """

        url = f"{ENDPOINT}/3/list/{list_id}/item_status"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def create_list(self, name, description="", language="", **kwargs):
        """
        POST /list
        """

        url = f"{ENDPOINT}/3/list"
        payload = {"name": name, "description": description, "language": language}
        return self.make_request("POST", url, kwargs, payload)

    @any_required_kwargs(["session_id"])
    @process_response
    def add_movie(self, list_id, media_id, **kwargs):
        """
        POST /list/{list_id}/add_item
        """

        url = f"{ENDPOINT}/3/list/{list_id}/add_item"
        payload = {"media_id": media_id}
        return self.make_request("POST", url, kwargs, payload)

    @any_required_kwargs(["session_id"])
    @process_response
    def remove_movie(self, list_id, media_id, **kwargs):
        """
        POST /list/{list_id}/remove_item
        """

        url = f"{ENDPOINT}/3/list/{list_id}/remove_item"
        payload = {"media_id": media_id}
        return self.make_request("POST", url, kwargs, payload)

    @any_required_kwargs(["session_id", "confirm"])
    @process_response
    def clear_list(self, list_id, **kwargs):
        """
        POST /list/{list_id}/clear
        """

        kwargs["confirm"] = str(kwargs["confirm"]).lower()
        url = f"{ENDPOINT}/3/list/{list_id}/clear"
        return self.make_request("POST", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def delete_list(self, list_id, **kwargs):
        """
        DELETE /list/{list_id}
        """

        url = f"{ENDPOINT}/3/list/{list_id}"
        return self.make_request("DELETE", url, kwargs)



from . import polite

keyword_id = 3586  # tv station


@polite
def test_get_keyword_details(tmdb):
    assert tmdb.get_keyword_details(keyword_id) is not None


@polite
def test_get_keyword_movies(tmdb):
    assert tmdb.get_keyword_movies(keyword_id) is not None



import pytest

from . import polite

search_types = ["company", "collection", "keyword", "movie", "multi", "person", "tv"]
search_kwargs = [({}, True), ({"query": "titanic"}, False)]


@polite
@pytest.mark.parametrize("search_type", search_types)
@pytest.mark.parametrize("kwargs, is_error", search_kwargs)
def test_get_search(tmdb, search_type, kwargs, is_error):
    method = getattr(tmdb, f"get_{search_type}_search")
    if is_error:
        with pytest.raises(RuntimeError):
            method(**kwargs)
    else:
        assert method(**kwargs) is not None



from .base import ENDPOINT, process_response


class GuestSessionsMixin:
    @process_response
    def get_guest_session_rated_movies(self, guest_session_id, **kwargs):
        """
        GET /guest_session/{guest_session_id}/rated/movies
        """

        url = f"{ENDPOINT}/3/guest_session/{guest_session_id}/rated/movies"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_guest_session_rated_tvs(self, guest_session_id, **kwargs):
        """
        GET /guest_session/{guest_session_id}/rated/tv
        """

        url = f"{ENDPOINT}/3/guest_session/{guest_session_id}/rated/tv"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_guest_session_rated_tv_episodes(self, guest_session_id, **kwargs):
        """
        GET /guest_session/{guest_session_id}/rated/tv/episodes
        """

        url = f"{ENDPOINT}/3/guest_session/{guest_session_id}/rated/tv/episodes"
        return self.make_request("GET", url, kwargs)



import os
import random
import time

import pytest

from . import polite

list_id = 14151
movie_id = 597


@polite
def test_get_list_details(tmdb):
    assert tmdb.get_list_details(list_id) is not None


@polite
@pytest.mark.parametrize("is_error", [True, False])
def test_get_list_item_status(tmdb, is_error):
    if is_error:
        with pytest.raises(RuntimeError):
            tmdb.get_list_item_status(list_id)
    else:
        assert tmdb.get_list_item_status(list_id, movie_id=movie_id) is not None


@polite
def test_list_operations(tmdb, session_id):
    list_id = tmdb.create_list("test_list", session_id=session_id).get("list_id")

    assert tmdb.add_movie(list_id, movie_id, session_id=session_id) is not None

    assert tmdb.remove_movie(list_id, movie_id, session_id=session_id) is not None

    assert tmdb.clear_list(list_id, session_id=session_id, confirm=True)

    try:
        tmdb.delete_list(list_id, session_id=session_id)
    except:
        # internal error happens quite often
        pass

    assert tmdb.delete_session(session_id) is not None



from os import path
from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md")) as f:
    long_description = f.read()

with open(path.join(here, "requirements.txt")) as f:
    requirements = f.read().splitlines()

setup(
    name="ezapi_tmdb",
    keywords="TMDb",
    version="0.6.2",
    packages=find_packages(),
    description="A Python wrapper for TMDb API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zehengl/ezapi-tmdb",
    author="Zeheng Li",
    author_email="imzehengl@gmail.com",
    maintainer="Zeheng Li",
    maintainer_email="imzehengl@gmail.com",
    license="MIT",
    install_requires=requirements,
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    test_suite="tests",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



from . import polite

company_id = 1  # Lucasfilm


@polite
def test_get_company_details(tmdb):
    assert tmdb.get_company_details(company_id) is not None


@polite
def test_get_company_alternative_names(tmdb):
    assert tmdb.get_company_alternative_names(company_id) is not None


@polite
def test_get_company_images(tmdb):
    assert tmdb.get_company_images(company_id) is not None



import requests
import wrapt

ENDPOINT = "https://api.themoviedb.org"


def any_required_kwargs(*required_kwargs_list):
    """
    Decorator to check if criteria of required kwargs are met

    Raises:
        RuntimeError: a bad client request

    Returns:
        func: decorated function
    """

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        compliances = [
            all([kwarg in kwargs for kwarg in required_kwargs])
            for required_kwargs in required_kwargs_list
        ]

        if any(compliances):
            return wrapped(*args, **kwargs)

        compliant_sets = [
            f'({" and ".join(required_kwargs)})'
            for required_kwargs in required_kwargs_list
        ]
        msg = f"required kwargs: {' or '.join(compliant_sets)}"

        raise RuntimeError(f"{msg}")

    return wrapper


@wrapt.decorator
def process_response(wrapped, instance, args, kwargs):
    """
    Decorator to process requests.Response

    Raises:
        Exception: Service Unavailable

    Returns:
        dict: json data
    """

    try:
        resp = wrapped(*args, **kwargs)

    except (requests.ConnectionError, requests.Timeout) as e:
        raise Exception("Service Unavailable") from e

    else:
        resp.raise_for_status()
        return resp.json()



from .base import ENDPOINT, any_required_kwargs, process_response


class MoviesMixin:
    @process_response
    def get_movie_details(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_movie_account_states(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/account_states
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/account_states"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_alternative_titles(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/alternative_titles
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/alternative_titles"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_changes(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/changes
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/changes"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_credits(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/credits
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/credits"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_external_ids(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/external_ids
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/external_ids"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_images(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/images
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/images"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_keywords(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/keywords
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/keywords"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_release_dates(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/release_dates
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/release_dates"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_videos(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/videos
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/videos"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_translations(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/translations
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/translations"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_recommendations(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/recommendations
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/recommendations"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_similar(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/similar
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/similar"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_reviews(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/reviews
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/reviews"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_lists(self, movie_id, **kwargs):
        """
        GET /movie/{movie_id}/lists
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/lists"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"], ["guest_session_id"])
    @process_response
    def post_movie_rating(self, movie_id, rating, **kwargs):
        """
        POST /movie/{movie_id}/rating
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/rating"
        payload = {"value": rating}
        return self.make_request("POST", url, kwargs, payload)

    @any_required_kwargs(["session_id"], ["guest_session_id"])
    @process_response
    def delete_movie_rating(self, movie_id, **kwargs):
        """
        DELETE /movie/{movie_id}/rating
        """

        url = f"{ENDPOINT}/3/movie/{movie_id}/rating"
        return self.make_request("DELETE", url, kwargs)

    @process_response
    def get_latest_movies(self, **kwargs):
        """
        GET /movie/latest
        """

        url = f"{ENDPOINT}/3/movie/latest"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_now_playing_movies(self, **kwargs):
        """
        GET /movie/now_playing
        """

        url = f"{ENDPOINT}/3/movie/now_playing"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_popular_movies(self, **kwargs):
        """
        GET /movie/popular
        """

        url = f"{ENDPOINT}/3/movie/popular"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_top_rated_movies(self, **kwargs):
        """
        GET /movie/top_rated
        """

        url = f"{ENDPOINT}/3/movie/top_rated"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_upcoming_movies(self, **kwargs):
        """
        GET /movie/upcoming
        """

        url = f"{ENDPOINT}/3/movie/upcoming"
        return self.make_request("GET", url, kwargs)



from . import polite


@polite
def test_get_movie_certifications(tmdb):
    assert tmdb.get_movie_certifications() is not None


@polite
def test_get_tv_certifications(tmdb):
    assert tmdb.get_tv_certifications() is not None



from .base import ENDPOINT, any_required_kwargs, process_response


class TVSeasonsMixin:
    @process_response
    def get_tv_season_details(self, tv_id, season_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_season_changes(self, season_id, **kwargs):
        """
        GET /tv/season/{season_id}/changes
        """

        url = f"{ENDPOINT}/3/tv/season/{season_id}/changes"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["session_id"])
    @process_response
    def get_tv_season_account_states(self, tv_id, season_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/account_states
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/account_states"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_season_credits(self, tv_id, season_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/credits
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/credits"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_season_external_ids(self, tv_id, season_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/external_ids
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/external_ids"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_season_images(self, tv_id, season_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/images
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/images"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_season_videos(self, tv_id, season_number, **kwargs):
        """
        GET /tv/{tv_id}/season/{season_number}/videos
        """

        url = f"{ENDPOINT}/3/tv/{tv_id}/season/{season_number}/videos"
        return self.make_request("GET", url, kwargs)
